# -*- coding: utf-8 -*-
__all__ = [
    "PYTHAINLP_DEFAULT_DATA_DIR",
    "get_full_data_path",
    "get_pythainlp_data_path",
    "get_pythainlp_path",
]

from pythainlp.tools.path import (
    PYTHAINLP_DEFAULT_DATA_DIR,
    get_full_data_path,
    get_pythainlp_data_path,
    get_pythainlp_path,
)
